{% extends 'viewflow/base_lockscreen.html' %}
{% load i18n viewflow %}

{% block title %}{{ title }} - {{ block.super }}{% endblock %}
{% block lockscreen-sidebar-header-title %}{{ title }}{% endblock %}


{% block lockscreen-sidebar-content %}
  <p>{% trans 'Forgotten your password? Enter your email address below, and we’ll email instructions for setting a new one.' %}</p>

  <vf-form>
    <form class="vf-form" method="POST" {% if form.is_multipart %} enctype="multipart/form-data"{% endif %}>
      {% csrf_token %}
      {% render form view.layout %}
      <div class="vf-page-lockscreen__sidebar-action-row">
        <a href="{% url 'login' %}" class="mdc-button mdc-card__action mdc-card__action--button" type="submit">{% trans 'Cancel' %}</a>
        <button class="mdc-button mdc-card__action mdc-card__action--button mdc-button--raised" type="submit">{% trans 'Reset' %}</button>
      </div>
    </form>
  </vf-form>
{% endblock %}

{% block lockscreen-content-icon %}<i class="logo__icon material-icons">lock</i>{% endblock %}
