{% load static %}
<script src="{% static 'viewflow/js/turbo.es2017-umd.js' %}" data-turbo-eval="false"></script>
<script src="{% static 'viewflow/js/perfect-scrollbar.min.js' %}"></script>
<script src="{% static 'viewflow/js/vis-network.min.js' %}"></script>
<script src="{% static 'viewflow/js/viewflow.min.js' %}"></script>
