{% load i18n static %}<!DOCTYPE html>
{% get_current_language as LANGUAGE_CODE %}{% get_current_language_bidi as LANGUAGE_BIDI %}
<html lang="{{ LANGUAGE_CODE|default:"en-us" }}" {% if LANGUAGE_BIDI %}dir="rtl"{% endif %}>
  {% with site=request.resolver_match.site app=request.resolver_match.app viewset=request.resolver_match.viewset %}
    <head>
      <meta charset="UTF-8">
      <meta name="viewport" content="width=device-width, initial-scale=1.0">
      {% block favicon %}<link rel="icon" href="{% static 'viewflow/img/favicon.png' %}" type="image/png" />{% endblock %}
      {% block turbo-cache-control %}<meta name="turbo-cache-control" content="no-cache">{% endblock %}
      <title>{% block title %}{{ site.title|default:"Django Material" }}{% endblock %}</title>
      {% block css %}
        {% include 'viewflow/includes/viewflow_css.html' %}
      {% endblock css %}
      {% block js %}
        {% url 'javascript-catalog' as javascript_catalog_url %}{% if javascript_catalog_url %}<script type="text/javascript" src="{{ javascript_catalog_url }}"></script>{% endif %}
        {% include 'viewflow/includes/viewflow_js.html' %}
      {% endblock js %}
      {% block extrahead %}{% endblock %}
    </head>
    <body>
      {% block theme %}
        {% if site.primary_color or site.secondary_color%}<style type="text/css">
          :root {
            {% if site.primary_color %}--mdc-theme-primary: {{ site.primary_color }};{% endif %}
            {% if site.secondary_color %}--mdc-theme-secondary: {{ site.secondary_color }};{% endif %}
          }
        </style>{% endif %}
      {% endblock %}
      <vf-turbo></vf-turbo>
      {% block body %}{% endblock %}
      {% include 'viewflow/includes/snackbar.html' %}
    </body>
  {% endwith %}
</html>
